#version 330

//shader constants
const float k0 = 0.1;	//constant attenuation
const float k1 = 0.2;	//linear attenuation
const float k2 = 3.0;	//quadratic attenuation

in vec2 fUV;
in vec3 fPosition;
in vec3 toCamera;
in mat3 TBN;

out vec4 color;

struct lightSource
{
  vec4 position;
  vec4 diffuse;
  float constantAttenuation, linearAttenuation, quadraticAttenuation;
  float spotCutoff, spotExponent;
  vec3 spotDirection;
};

lightSource light0 = lightSource(
	vec4(30, 0,  30, 1.0),
	vec4(1.0, 1.0, 1.0, 1.0),
	0.0001, 0.0002, 0.002,
	80.0, 20.0,
	vec3(-1.0, -0.5, 1.0)
);

lightSource light1 = lightSource(
	vec4(15,  15,  5, 1.0),
	vec4(1.0, 1.0, 1.0, 1.0),
	0.5, 1.0, 0.2,
	80.0, 20.0,
	vec3(-1.0, 0.5, -1.0)
);

struct Material
{
	vec3 ambientColor;
	vec3 diffuseColor;
	vec3 specularColor;
	vec3 emissiveColor;
	vec3 transparentColor;
	vec3 reflectiveColor;

	float shininess;
	float shininessStrength;
	float reflectivity;
	float opacity;
};

uniform sampler2D tex;
uniform sampler2D normalMap;

uniform mat4 modelMatrix;
uniform mat4 viewMatrix;
uniform vec3 intensities;

uniform int bOn;
uniform vec4 lightpos;
uniform vec3 lightdir;

uniform Material material;

void main() {
	
	vec3 totalLighting;

	vec3 positionToLightSource = vec3(light0.position) - fPosition;

	float distance = length(positionToLightSource);
	vec3 lightDirection = normalize(positionToLightSource);
	
	float attenuation = 1.0 / (light0.constantAttenuation
				+ light0.linearAttenuation * distance
				+ light0.quadraticAttenuation * distance * distance);

	//calculate new normal using normalmaps
	//http://www.opengl-tutorial.org/intermediate-tutorials/tutorial-13-normal-mapping/
	vec3 newNormal =  normalize(TBN * (texture2D(normalMap, fUV).rgb*2.0 - vec3(1.0f, 1.0f, 1.0f)));

	vec3 diffuseReflection = attenuation * vec3(light0.diffuse) * material.diffuseColor * max(0.0, dot(newNormal, lightDirection));

	vec3 specularReflection = vec3(0.0, 0.0, 0.0);
	if (dot(newNormal, lightDirection) < 0.0){
			// light source on the wrong side?
			specularReflection = vec3(0.0, 0.0, 0.0); // no specular reflection
	}else{
		// light source on the right side
		//  specularReflection = attenuation * vec3(light0.specular) * vec3(material.specularColor) * pow(max(0.0, dot(reflect(-lightDirection, newNormal), viewDirection)), material.shininess);
	}

	totalLighting = material.ambientColor +  diffuseReflection + specularReflection;

	color = vec4(totalLighting,material.opacity) * texture(tex,fUV);
	
	/*FLASHLIGHT*/

	if(bOn == 1) {
		vec3 L = normalize(vec3(lightpos) - fPosition);

		float dist = length(L);

		L = normalize(L);

		//float diffuse = max( 0, dot(newNormal,L));

		float attenuationAmount = 1.0/(k0 + k1*dist + k2*dist*dist);

		vec3 spotdirect = normalize(lightdir);

		vec3 D = -L;

		float spotEffect = dot(D, spotdirect);

		if(spotEffect > 0.97f) {
			float diffuse = max(0, dot(newNormal, L));

			spotEffect = pow(spotEffect, 1.2);

			attenuationAmount *= spotEffect;
			diffuse *= attenuationAmount;

			vec4 addColor = diffuse * texture(tex,fUV);
			addColor.w=0.0f;

			color += addColor;
		} else if(spotEffect >= 0.95f) {
			float diffuse = max(0, dot(newNormal, L));

			float fvalue = clamp(50.f*spotEffect-47.5f,0.0f,1.0f);

			spotEffect = pow(spotEffect, 1.2);

			attenuationAmount *= spotEffect;
			diffuse *= attenuationAmount;

			vec4 addColor = diffuse * fvalue * texture(tex,fUV);
			addColor.w=0.0f;

			color += addColor;
		}

	}
}